--
-- Jan & Uwe R. Zimmer, Australia, July 2011
--

with Swarm_Control;    use Swarm_Control;
with Swarm_Data;       use Swarm_Data;
with Swarm_Structures; use Swarm_Structures;

package body Swarm_Control_Concurrent is

   --

   protected Synchroniser is
      entry All_Jobs_Done;
   private
      Synchroniser_Open : Boolean := False;
   end Synchroniser;

   --

   protected body Synchroniser is
      entry All_Jobs_Done when
        All_Jobs_Done'Count = No_Of_CPU_Cores + 1 or
        Synchroniser_Open is
      begin
         Synchroniser_Open := All_Jobs_Done'Count > 0;
      end All_Jobs_Done;
   end Synchroniser;

   --
   --
   --

   task type Worker is
      entry Set_Job (Job : Job_Type; Start_Index, End_Index : Swarm_Element_Index);
   end Worker;

   --

   Workers : Array (1 .. No_Of_CPU_Cores) of Worker;

   --

   task body Worker is

      Current_Job   : Job_Type;
      Start, Finish : Swarm_Element_Index;

   begin
      loop
         select
            accept Set_Job (Job : Job_Type; Start_Index, End_Index : Swarm_Element_Index) do
               Current_Job := Job;
               Start       := Start_Index;
               Finish      := End_Index;
            end Set_Job;
            for Element_Index in Start .. Finish loop
               case Current_Job is
                  when Set_Acceleration => Set_Acceleration (Element_Index);
                  when Move_Element     => Move_Element     (Element_Index);
               end case;
            end loop;
            Synchroniser.All_Jobs_Done;
         or
            terminate;
         end select;
      end loop;
   end Worker;


   --------------------------------------
   -- Distribute_Jobs --
   --------------------------------------

   procedure Distribute_Jobs (Job : Job_Type) is

      use Swarm_Vectors;

      First_Element     : constant Swarm_Element_Index := First_Index (Swarm_State);
      Last_Element      : constant Swarm_Element_Index := Last_Index  (Swarm_State);
      No_Of_Elements    : constant Swarm_Element_Index := Swarm_Element_Index (Length (Swarm_State));
      Elements_Per_Task : constant Swarm_Element_Index := No_Of_Elements / No_Of_CPU_Cores;

   begin
      for Task_Index in Workers'First .. Workers'Last - 1 loop

         Workers (Task_Index).Set_Job
           (Job,
            Swarm_Element_Index ((Integer (Task_Index) - Workers'First    ) * Elements_Per_Task + First_Element),
            Swarm_Element_Index ((Integer (Task_Index) - Workers'First + 1) * Elements_Per_Task + First_Element - 1));

      end loop;

      Workers (Workers'Last).Set_Job
        (Set_Acceleration,
         Swarm_Element_Index ((Integer (Workers'Last) - Workers'First    ) * Elements_Per_Task + First_Element),
         Last_Element);

      Synchroniser.All_Jobs_Done;
   end Distribute_Jobs;

   --

end Swarm_Control_Concurrent;
